/*************************************************************************
DESCRIPTION :
The TTCsynchronizer crosses the clock domanins from TTC_CK40Des1 to clk.
The input pulse must be longer than 12.5 ns for the circuit to work.
This covers the ample jitter of the TTC signals (25ns +/- jitter).
The resulting output signal is a 1-clk-tick pulses.
***************************************************************************/

`timescale 1ns / 1ns
module TTCsynchronizer  (clk, clk2x, t_Command, Command);

 input   clk, clk2x;
 input   t_Command;  
 output  Command;	    reg  Command;	

 reg Command_D,  Command_2D, Command_3D, Command_clk2x;
 reg Command_4D, Command_5D; 
 
 always @ (posedge clk2x)    // digital edge-detection of t_Command
   begin 
     Command_D     <= t_Command;
     Command_2D    <= Command_D;
     Command_3D    <= Command_2D;
     Command_clk2x <= Command_D | Command_2D | Command_3D;// ultra-conservative
   end

 always @ (posedge clk )   begin   
    // detect transition 0->1 and make a 25-ns pulse
    Command_4D <= Command_clk2x;    
    Command_5D <= Command_4D;  
    Command    <= Command_4D  & ~Command_5D;   
  end  
    
endmodule  