`include "define.v"
module slbsTop  ( 

 input  CLKPLUS, CLKMINUS, // 40MHz differential RX_CLK
 input  TTC_CK40Des1, TXNRX, TTC_BRCST_STRB1, TTC_BCR,
 input  LOC_C_S,  LOC_R_W, 
 input  [7:2] TTC_BRCST,      
 input  [8:0] SYNC1, SYNC2, SYNC3, SYNC4, 
              SYNC5, SYNC6, SYNC7, SYNC8,   // Data In
 input  [`LB_Addr_Size-1:0]  LOCADD,  // The Address bus.
 inout  [`REG_WIDTH-1:0]     LOCBUS,  // The bidir Data bus 

 output     TPMINUS, TPPLUS,
 output     TRSTN,     REFCLK,
 output reg PECL_OUT         );

  wire run_status,  BunchEr, GateTTCcommands, StartStop_Spy, 
       CounterModeSel, PatternMode, RepeatMode,
       TTC_Hard_rst, TTC_Soft_rst, TTC_Resync, 
       TTC_Start, TTC_Stop, TTC_AsyncFifoRst,
       VME_Hard_rst, VME_Soft_rst, 
       VME_Start, VME_Stop, VME_AsyncFifoRst,
       Hard_rst, Soft_rst,  Start, Stop;
  wire [7:0] LastTTCcommand;
  wire [15:0] TTC_test0, TTC_test1, TTC_test2, TTC_test3,
              TTC_test4, TTC_test5, TTC_test6, TTC_test7,
              TTC_test8, TTC_test9, TTC_testA, TTC_testB,
              TTC_testC, TTC_testD, TTC_testE, TTC_testF,
   Pattern1Back, Pattern2Back, Pattern3Back, Pattern4Back,
   Pattern5Back, Pattern6Back, Pattern7Back, Pattern8Back,
   SnapFIFO1,  SnapFIFO2,  SnapFIFO3, SnapFIFO4,
   SnapFIFO5,  SnapFIFO6,  SnapFIFO7, SnapFIFO8;

  wire [`BCN_WIDTH-1:0] MaxBCN; 
  wire [`REG_WIDTH-1:0] SlbsID; 
  wire [8:1] PatternsWrEn, SnapRdEn;   
  wire [23:0] ChMask;        
  wire [2:0]  LB_STATE;       
  wire [`REG_WIDTH-1:0] Dout, Din; 
  reg  [8:1] And, Or;  

//  ********************************************************************
//      ********************  CLOCK SECTION    *****************
//            *********************************************

// XST was stupid at handling clocks and clock buffer, so we instantiate
// all the buffers and inihbit XST from changing them with the attribute.
 wire   RxClk, SelectedClk, clk, clk2x, clk0, clk2x_dll;
 wire   [7:0] DLL_status;   // DLL_status[1] asserted if .CLKIN stopped
 wire   DLL_lock, ClkFx, ClkFx180, clk3x, clk3x180;
 reg    ResetDLL, DLL_lock_D;
 reg    [14:0] RstCnt; // counter for proper reset of DLL 
 reg    [2:0] UnlockCnt; // counter for proper reset of DLL 

 // Instantiate the Differential Global Input Buffer 
  IBUFGDS_LVPECL_25 difclkbuf (.I(CLKPLUS), .IB(CLKMINUS), .O(RxClk));
  IBUFG ttcclkbuf ( .I(TTC_CK40Des1), .O(TTC_CK) );

  BUFGMUX SystemClockMux 
   ( .I0 (TTC_CK),      .I1 (RxClk),
     .O (SelectedClk),  .S (ClockSelect)     );

// Generate a DLL reset (3-ticks) after FPGA is configured, see:
// www.xilinx.com/publications/xcellonline/xcell_51/xc_particle51.htm
  FDS flop1 (.D(1'b0), .C(SelectedClk), .Q(out1), .S(1'b0)); 
  FD  flop2 (.D(out1), .C(SelectedClk), .Q(out2)); 
  FD  flop3 (.D(out2), .C(SelectedClk), .Q(out3)); 
  FD  flop4 (.D(out3), .C(SelectedClk), .Q(out4)); 

  assign ConfigRst = (out2 | out3 | out4); 

  DCM MainDCM
   (.CLKIN(SelectedClk), .CLKFB(clk),  .RST(ConfigRst),  
    .CLK0(clk0),   .CLK90(),  .CLK180(),  .CLK270(),
    .CLK2X(clk2x_dll),  .CLK2X180(),
    .CLKDV(),   .CLKFX(ClkFx),   .CLKFX180(ClkFx180),
    .LOCKED(DLL_lock),  .STATUS(DLL_status),
    .DSSEN(1'b0),
    .PSCLK(1'b0),  .PSEN(1'b0),  .PSINCDEC(1'b0), .PSDONE() );

  //synthesis attribute DFS_FREQUENCY_MODE of MainDCM is "LOW" 
  //synthesis attribute CLKFX_DIVIDE of MainDCM is "1" 
  //synthesis attribute CLKFX_MULTIPLY of MainDCM is "3" 
  //synthesis attribute STARTUP_WAIT of MainDCM is "FALSE" 

/* If tests show that DCM looses lock, consider to
   increase the FACTORY_JF setting to 'FFFF  (Xilinx Answer 13756).
   Anyway lab test in Dec03 showed that the DLL relock spontaneously
   when the TTCrx was loosing lock and then relocking   */
/* If there is the chance that SelectedClk stops completly, the LOCKED 
   signal will not update; consider the STATUS bit "CLK_IN_STOPPED".
   I do not implement it as I don't expect this occurence, although
   trivial mechanical troubles of the TTC-PMC mezzanine could do it. 
   Note that if the TTCrx looses lock, TTC_clk will keep toggling at 
   the TTCrx natural oscillation frequency, but will not stop.    */

// ResetDLL has some "feedback" problems when connected to DCM,
// so far I don't use it.
 always @ (posedge TTC_CK) begin  //TTC_CK should toggle even if no cable 
    if (DLL_lock == 0) begin
         RstCnt <= RstCnt + 1;
         if (RstCnt == 15'h 7FFF)
               ResetDLL <= 1;  // reset no more then every 2^15 25ns
         else     ResetDLL <= 0;
    end
    DLL_lock_D <= DLL_lock;
    if (Hard_rst)  UnlockCnt <= 0;
    else if (DLL_lock_D & (~DLL_lock) & (UnlockCnt != 3'b111)) //hi->lo transition
             UnlockCnt <= UnlockCnt + 1;   // increment (stop at max count)
 end //always 

 BUFG CK40BUF (.I(clk0), .O(clk));  // Use clk as internal clock 
 BUFG CK2XBUF (.I(clk2x_dll), .O(clk2x)); 
 BUFG CK3XBUF    (.I(ClkFx),    .O(clk3x));
 BUFG CK3X180BUF (.I(ClkFx180), .O(clk3x180));

 assign REFCLK = clk3x; // to U2A and then to Wisc_Mezzanine.

//       *************  END OF CLOCK SECTION  *****************
//  *****************************************************************

 assign TRSTN = 1'b1; // Wisc. Rec_Mezz JTAG reset.

// test all input data lines
 always@(posedge clk)  begin  
    And[1] <= &SYNC1;     And[2] <= &SYNC2; 
    And[3] <= &SYNC3;     And[4] <= &SYNC4; 
    And[5] <= &SYNC5;     And[6] <= &SYNC6; 
    And[7] <= &SYNC7;     And[8] <= &SYNC8; 
    Or[1]  <= &SYNC1;     Or[2]  <= &SYNC2; 
    Or[3]  <= &SYNC3;     Or[4]  <= &SYNC4; 
    Or[5]  <= &SYNC5;     Or[6]  <= &SYNC6; 
    Or[7]  <= &SYNC7;     Or[8]  <= &SYNC8; 

    PECL_OUT <= &And & TXNRX; 
  end

 OBUFDS_LVPECL_25 RJ45pair (.I(|Or), .O(TPPLUS), .OB(TPMINUS));

LocBusSlave LocBusSlave (
    .clk(clk),       .rst(Hard_rst),  .ClockSelect(ClockSelect),
    .LocAddr(LOCADD),    .LocData(LOCBUS), 
    .LocCS_b(LOC_C_S),    .LocWRITE_b(LOC_R_W), 
    .LB_STATE(LB_STATE),    .Din(Din), 
    .Dout(Dout),            .Tri(), 
    .DLL_lock(DLL_lock),    .UnlockCnt(UnlockCnt), 
    .LastTTCcommand(LastTTCcommand), .GateTTCcommands(GateTTCcommands), 
    .VME_Hard_rst(VME_Hard_rst),     .VME_Soft_rst(VME_Soft_rst), 
    .VME_Start(VME_Start),           .VME_Stop(VME_Stop), 
    .VME_AsyncFifoRst(VME_AsyncFifoRst), 
    .PatternMode(PatternMode),      .RepeatMode(RepeatMode), 
    .run_status(run_status),     .StartStop_Spy(StartStop_Spy), 
    .SlbsID(SlbsID),      .ChMask(ChMask), 
    .SnapRdEn(SnapRdEn), 
    .SnapFIFO1(SnapFIFO1),       .SnapFIFO2(SnapFIFO2), 
    .SnapFIFO3(SnapFIFO3),       .SnapFIFO4(SnapFIFO4), 
    .SnapFIFO5(SnapFIFO5),       .SnapFIFO6(SnapFIFO6), 
    .SnapFIFO7(SnapFIFO7),       .SnapFIFO8(SnapFIFO8), 
    .TTC_test0(TTC_test0),       .TTC_test1(TTC_test1), 
    .TTC_test2(TTC_test2),       .TTC_test3(TTC_test3), 
    .TTC_test4(TTC_test4),       .TTC_test5(TTC_test5), 
    .TTC_test6(TTC_test6),       .TTC_test7(TTC_test7), 
    .TTC_test8(TTC_test8),       .TTC_test9(TTC_test9), 
    .TTC_testA(TTC_testA),       .TTC_testB(TTC_testB), 
    .TTC_testC(TTC_testC),       .TTC_testD(TTC_testD), 
    .TTC_testE(TTC_testE),       .TTC_testF(TTC_testF), 
    .MaxBCN(MaxBCN),             .BunchEr(BunchEr), 
    .PatternsWrEn(PatternsWrEn), 
    .Pattern1Back(Pattern1Back),     .Pattern2Back(Pattern2Back), 
    .Pattern3Back(Pattern3Back),     .Pattern4Back(Pattern4Back), 
    .Pattern5Back(Pattern5Back),     .Pattern6Back(Pattern6Back), 
    .Pattern7Back(Pattern7Back),     .Pattern8Back(Pattern8Back), 
    .CounterModeSel(CounterModeSel)    );

  TTC_BrcstDecoder TTC_BrcstDecoder (
   .clk(clk),      .clk2x(clk2x),       .reset(VME_Hard_rst),
   .tTTC_BCntRes(TTC_BCR),              .TTC_Brcst(TTC_BRCST),
   .TTC_BrcstStr(TTC_BRCST_STRB1),      .TTC_CK40Des1(TTC_CK),  
   .LastTTCcommand(LastTTCcommand),     .TTC_Resync(TTC_Resync),
   .TTC_Hard_rst(TTC_Hard_rst),         .TTC_Soft_rst(TTC_Soft_rst),  
   .TTC_Start(TTC_Start),               .TTC_Stop(TTC_Stop),
   .TTC_BC0(TTC_BC0),                  
   .TTC_AsyncFifoRst(TTC_AsyncFifoRst), .TTC_TestEnable(TTC_TestEnable),
   .TTC_test0(TTC_test0), .TTC_test1(TTC_test1), .TTC_test2(TTC_test2), 
   .TTC_test3(TTC_test3), .TTC_test4(TTC_test4), .TTC_test5(TTC_test5),
   .TTC_test6(TTC_test6), .TTC_test7(TTC_test7), .TTC_test8(TTC_test8),
   .TTC_test9(TTC_test9), .TTC_testA(TTC_testA), .TTC_testB(TTC_testB),
   .TTC_testC(TTC_testC), .TTC_testD(TTC_testD), .TTC_testE(TTC_testE),
   .TTC_testF(TTC_testF),   .MaxBCN(MaxBCN),   .BunchEr(BunchEr),
   .BCN(),   .OrN() );

 commander commander (  
  .clk(clk),       .TTC_Resync(TTC_Resync), 
  .GateTTCcommands(GateTTCcommands),
  .TTC_Hard_rst(TTC_Hard_rst), .TTC_Soft_rst(TTC_Soft_rst),
  .TTC_Start(TTC_Start), .TTC_Stop(TTC_Stop), 
  .TTC_AsyncFifoRst(TTC_AsyncFifoRst),
  .VME_Hard_rst(VME_Hard_rst), .VME_Soft_rst(VME_Soft_rst), 
  .VME_Start(VME_Start), .VME_Stop(VME_Stop), .VME_AsyncFifoRst(VME_AsyncFifoRst),
  .Hard_rst(Hard_rst),  .Soft_rst(Soft_rst), 
  .Start(Start), .Stop(Stop),  .AsyncFifoRst(AsyncFifoRst),
  .run_status(run_status)       );

endmodule